# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.

from typing import Any, Dict
from dateutil import parser

from scaleway_core.profile import ProfileDefaults
from scaleway_core.utils import (
    OneOfPossibility,
    resolve_one_of,
)
from .types import (
    Link,
    Partner,
    Pop,
    RoutingPolicy,
    ListLinksResponse,
    ListPartnersResponse,
    ListPopsResponse,
    ListRoutingPoliciesResponse,
    AttachRoutingPolicyRequest,
    AttachVpcRequest,
    CreateLinkRequest,
    CreateRoutingPolicyRequest,
    UpdateLinkRequest,
    UpdateRoutingPolicyRequest,
)


def unmarshal_Link(data: Any) -> Link:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'Link' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("id", None)
    if field is not None:
        args["id"] = field

    field = data.get("project_id", None)
    if field is not None:
        args["project_id"] = field

    field = data.get("organization_id", None)
    if field is not None:
        args["organization_id"] = field

    field = data.get("name", None)
    if field is not None:
        args["name"] = field

    field = data.get("tags", None)
    if field is not None:
        args["tags"] = field

    field = data.get("pop_id", None)
    if field is not None:
        args["pop_id"] = field

    field = data.get("bandwidth_mbps", None)
    if field is not None:
        args["bandwidth_mbps"] = field

    field = data.get("partner_id", None)
    if field is not None:
        args["partner_id"] = field
    else:
        args["partner_id"] = None

    field = data.get("status", None)
    if field is not None:
        args["status"] = field

    field = data.get("bgp_v4_status", None)
    if field is not None:
        args["bgp_v4_status"] = field

    field = data.get("bgp_v6_status", None)
    if field is not None:
        args["bgp_v6_status"] = field

    field = data.get("enable_route_propagation", None)
    if field is not None:
        args["enable_route_propagation"] = field

    field = data.get("pairing_key", None)
    if field is not None:
        args["pairing_key"] = field

    field = data.get("region", None)
    if field is not None:
        args["region"] = field

    field = data.get("vpc_id", None)
    if field is not None:
        args["vpc_id"] = field
    else:
        args["vpc_id"] = None

    field = data.get("routing_policy_id", None)
    if field is not None:
        args["routing_policy_id"] = field
    else:
        args["routing_policy_id"] = None

    field = data.get("created_at", None)
    if field is not None:
        args["created_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["created_at"] = None

    field = data.get("updated_at", None)
    if field is not None:
        args["updated_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["updated_at"] = None

    field = data.get("disapproved_reason", None)
    if field is not None:
        args["disapproved_reason"] = field
    else:
        args["disapproved_reason"] = None

    return Link(**args)


def unmarshal_Partner(data: Any) -> Partner:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'Partner' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("id", None)
    if field is not None:
        args["id"] = field

    field = data.get("name", None)
    if field is not None:
        args["name"] = field

    field = data.get("contact_email", None)
    if field is not None:
        args["contact_email"] = field

    field = data.get("logo_url", None)
    if field is not None:
        args["logo_url"] = field

    field = data.get("portal_url", None)
    if field is not None:
        args["portal_url"] = field

    field = data.get("created_at", None)
    if field is not None:
        args["created_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["created_at"] = None

    field = data.get("updated_at", None)
    if field is not None:
        args["updated_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["updated_at"] = None

    return Partner(**args)


def unmarshal_Pop(data: Any) -> Pop:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'Pop' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("id", None)
    if field is not None:
        args["id"] = field

    field = data.get("name", None)
    if field is not None:
        args["name"] = field

    field = data.get("hosting_provider_name", None)
    if field is not None:
        args["hosting_provider_name"] = field

    field = data.get("address", None)
    if field is not None:
        args["address"] = field

    field = data.get("city", None)
    if field is not None:
        args["city"] = field

    field = data.get("logo_url", None)
    if field is not None:
        args["logo_url"] = field

    field = data.get("available_link_bandwidths_mbps", None)
    if field is not None:
        args["available_link_bandwidths_mbps"] = field

    field = data.get("region", None)
    if field is not None:
        args["region"] = field

    return Pop(**args)


def unmarshal_RoutingPolicy(data: Any) -> RoutingPolicy:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'RoutingPolicy' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("id", None)
    if field is not None:
        args["id"] = field

    field = data.get("project_id", None)
    if field is not None:
        args["project_id"] = field

    field = data.get("organization_id", None)
    if field is not None:
        args["organization_id"] = field

    field = data.get("name", None)
    if field is not None:
        args["name"] = field

    field = data.get("tags", None)
    if field is not None:
        args["tags"] = field

    field = data.get("prefix_filter_in", None)
    if field is not None:
        args["prefix_filter_in"] = field

    field = data.get("prefix_filter_out", None)
    if field is not None:
        args["prefix_filter_out"] = field

    field = data.get("region", None)
    if field is not None:
        args["region"] = field

    field = data.get("created_at", None)
    if field is not None:
        args["created_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["created_at"] = None

    field = data.get("updated_at", None)
    if field is not None:
        args["updated_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["updated_at"] = None

    return RoutingPolicy(**args)


def unmarshal_ListLinksResponse(data: Any) -> ListLinksResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ListLinksResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("links", None)
    if field is not None:
        args["links"] = (
            [unmarshal_Link(v) for v in field] if field is not None else None
        )

    field = data.get("total_count", None)
    if field is not None:
        args["total_count"] = field

    return ListLinksResponse(**args)


def unmarshal_ListPartnersResponse(data: Any) -> ListPartnersResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ListPartnersResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("partners", None)
    if field is not None:
        args["partners"] = (
            [unmarshal_Partner(v) for v in field] if field is not None else None
        )

    field = data.get("total_count", None)
    if field is not None:
        args["total_count"] = field

    return ListPartnersResponse(**args)


def unmarshal_ListPopsResponse(data: Any) -> ListPopsResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ListPopsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("pops", None)
    if field is not None:
        args["pops"] = [unmarshal_Pop(v) for v in field] if field is not None else None

    field = data.get("total_count", None)
    if field is not None:
        args["total_count"] = field

    return ListPopsResponse(**args)


def unmarshal_ListRoutingPoliciesResponse(data: Any) -> ListRoutingPoliciesResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ListRoutingPoliciesResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("routing_policies", None)
    if field is not None:
        args["routing_policies"] = (
            [unmarshal_RoutingPolicy(v) for v in field] if field is not None else None
        )

    field = data.get("total_count", None)
    if field is not None:
        args["total_count"] = field

    return ListRoutingPoliciesResponse(**args)


def marshal_AttachRoutingPolicyRequest(
    request: AttachRoutingPolicyRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.routing_policy_id is not None:
        output["routing_policy_id"] = request.routing_policy_id

    return output


def marshal_AttachVpcRequest(
    request: AttachVpcRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.vpc_id is not None:
        output["vpc_id"] = request.vpc_id

    return output


def marshal_CreateLinkRequest(
    request: CreateLinkRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}
    output.update(
        resolve_one_of(
            [
                OneOfPossibility("dedicated", request.dedicated),
                OneOfPossibility("port_id", request.port_id),
                OneOfPossibility("partner_id", request.partner_id),
            ]
        ),
    )

    if request.name is not None:
        output["name"] = request.name

    if request.pop_id is not None:
        output["pop_id"] = request.pop_id

    if request.bandwidth_mbps is not None:
        output["bandwidth_mbps"] = request.bandwidth_mbps

    if request.project_id is not None:
        output["project_id"] = request.project_id or defaults.default_project_id

    if request.tags is not None:
        output["tags"] = request.tags

    return output


def marshal_CreateRoutingPolicyRequest(
    request: CreateRoutingPolicyRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.name is not None:
        output["name"] = request.name

    if request.project_id is not None:
        output["project_id"] = request.project_id or defaults.default_project_id

    if request.tags is not None:
        output["tags"] = request.tags

    if request.prefix_filter_in is not None:
        output["prefix_filter_in"] = request.prefix_filter_in

    if request.prefix_filter_out is not None:
        output["prefix_filter_out"] = request.prefix_filter_out

    return output


def marshal_UpdateLinkRequest(
    request: UpdateLinkRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.name is not None:
        output["name"] = request.name

    if request.tags is not None:
        output["tags"] = request.tags

    return output


def marshal_UpdateRoutingPolicyRequest(
    request: UpdateRoutingPolicyRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.name is not None:
        output["name"] = request.name

    if request.tags is not None:
        output["tags"] = request.tags

    if request.prefix_filter_in is not None:
        output["prefix_filter_in"] = request.prefix_filter_in

    if request.prefix_filter_out is not None:
        output["prefix_filter_out"] = request.prefix_filter_out

    return output



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.

from typing import Any, Dict
from dateutil import parser

from scaleway_core.profile import ProfileDefaults
from .types import (
    EmailFlag,
    WebhookEventType,
    EmailTry,
    Email,
    DomainRecordsDMARC,
    DomainRecords,
    DomainReputation,
    DomainStatistics,
    Domain,
    OfferSubscription,
    Webhook,
    Blocklist,
    BulkCreateBlocklistsResponse,
    CreateEmailResponse,
    DomainLastStatusAutoconfigState,
    DomainLastStatusDkimRecord,
    DomainLastStatusDmarcRecord,
    DomainLastStatusSpfRecord,
    DomainLastStatus,
    ListBlocklistsResponse,
    ListDomainsResponse,
    ListEmailsResponse,
    ListOfferSubscriptionsResponse,
    Offer,
    ListOffersResponse,
    Pool,
    ListPoolsResponse,
    WebhookEvent,
    ListWebhookEventsResponse,
    ListWebhooksResponse,
    ProjectConsumption,
    ProjectSettingsPeriodicReport,
    ProjectSettings,
    Statistics,
    BulkCreateBlocklistsRequest,
    CreateDomainRequest,
    CreateEmailRequestAddress,
    CreateEmailRequestAttachment,
    CreateEmailRequestHeader,
    CreateEmailRequest,
    CreateWebhookRequest,
    UpdateDomainRequest,
    UpdateOfferSubscriptionRequest,
    UpdateProjectSettingsRequestUpdatePeriodicReport,
    UpdateProjectSettingsRequest,
    UpdateWebhookRequest,
)


def unmarshal_EmailTry(data: Any) -> EmailTry:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'EmailTry' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("rank", None)
    if field is not None:
        args["rank"] = field

    field = data.get("code", None)
    if field is not None:
        args["code"] = field

    field = data.get("message", None)
    if field is not None:
        args["message"] = field

    field = data.get("tried_at", None)
    if field is not None:
        args["tried_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["tried_at"] = None

    return EmailTry(**args)


def unmarshal_Email(data: Any) -> Email:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'Email' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("id", None)
    if field is not None:
        args["id"] = field

    field = data.get("message_id", None)
    if field is not None:
        args["message_id"] = field

    field = data.get("project_id", None)
    if field is not None:
        args["project_id"] = field

    field = data.get("mail_from", None)
    if field is not None:
        args["mail_from"] = field

    field = data.get("mail_rcpt", None)
    if field is not None:
        args["mail_rcpt"] = field

    field = data.get("rcpt_to", None)
    if field is not None:
        args["rcpt_to"] = field
    else:
        args["rcpt_to"] = None

    field = data.get("rcpt_type", None)
    if field is not None:
        args["rcpt_type"] = field

    field = data.get("subject", None)
    if field is not None:
        args["subject"] = field

    field = data.get("status", None)
    if field is not None:
        args["status"] = field

    field = data.get("try_count", None)
    if field is not None:
        args["try_count"] = field

    field = data.get("last_tries", None)
    if field is not None:
        args["last_tries"] = (
            [unmarshal_EmailTry(v) for v in field] if field is not None else None
        )

    field = data.get("flags", None)
    if field is not None:
        args["flags"] = [EmailFlag(v) for v in field] if field is not None else None

    field = data.get("created_at", None)
    if field is not None:
        args["created_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["created_at"] = None

    field = data.get("updated_at", None)
    if field is not None:
        args["updated_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["updated_at"] = None

    field = data.get("status_details", None)
    if field is not None:
        args["status_details"] = field
    else:
        args["status_details"] = None

    return Email(**args)


def unmarshal_DomainRecordsDMARC(data: Any) -> DomainRecordsDMARC:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'DomainRecordsDMARC' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("name", None)
    if field is not None:
        args["name"] = field

    field = data.get("value", None)
    if field is not None:
        args["value"] = field

    return DomainRecordsDMARC(**args)


def unmarshal_DomainRecords(data: Any) -> DomainRecords:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'DomainRecords' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("dmarc", None)
    if field is not None:
        args["dmarc"] = unmarshal_DomainRecordsDMARC(field)
    else:
        args["dmarc"] = None

    return DomainRecords(**args)


def unmarshal_DomainReputation(data: Any) -> DomainReputation:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'DomainReputation' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("status", None)
    if field is not None:
        args["status"] = field

    field = data.get("score", None)
    if field is not None:
        args["score"] = field

    field = data.get("scored_at", None)
    if field is not None:
        args["scored_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["scored_at"] = None

    field = data.get("previous_score", None)
    if field is not None:
        args["previous_score"] = field
    else:
        args["previous_score"] = None

    field = data.get("previous_scored_at", None)
    if field is not None:
        args["previous_scored_at"] = (
            parser.isoparse(field) if isinstance(field, str) else field
        )
    else:
        args["previous_scored_at"] = None

    return DomainReputation(**args)


def unmarshal_DomainStatistics(data: Any) -> DomainStatistics:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'DomainStatistics' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("total_count", None)
    if field is not None:
        args["total_count"] = field

    field = data.get("sent_count", None)
    if field is not None:
        args["sent_count"] = field

    field = data.get("failed_count", None)
    if field is not None:
        args["failed_count"] = field

    field = data.get("canceled_count", None)
    if field is not None:
        args["canceled_count"] = field

    return DomainStatistics(**args)


def unmarshal_Domain(data: Any) -> Domain:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'Domain' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("id", None)
    if field is not None:
        args["id"] = field

    field = data.get("organization_id", None)
    if field is not None:
        args["organization_id"] = field

    field = data.get("project_id", None)
    if field is not None:
        args["project_id"] = field

    field = data.get("name", None)
    if field is not None:
        args["name"] = field

    field = data.get("status", None)
    if field is not None:
        args["status"] = field

    field = data.get("spf_config", None)
    if field is not None:
        args["spf_config"] = field

    field = data.get("dkim_config", None)
    if field is not None:
        args["dkim_config"] = field

    field = data.get("created_at", None)
    if field is not None:
        args["created_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["created_at"] = None

    field = data.get("next_check_at", None)
    if field is not None:
        args["next_check_at"] = (
            parser.isoparse(field) if isinstance(field, str) else field
        )
    else:
        args["next_check_at"] = None

    field = data.get("last_valid_at", None)
    if field is not None:
        args["last_valid_at"] = (
            parser.isoparse(field) if isinstance(field, str) else field
        )
    else:
        args["last_valid_at"] = None

    field = data.get("autoconfig", None)
    if field is not None:
        args["autoconfig"] = field

    field = data.get("region", None)
    if field is not None:
        args["region"] = field

    field = data.get("revoked_at", None)
    if field is not None:
        args["revoked_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["revoked_at"] = None

    field = data.get("last_error", None)
    if field is not None:
        args["last_error"] = field
    else:
        args["last_error"] = None

    field = data.get("statistics", None)
    if field is not None:
        args["statistics"] = unmarshal_DomainStatistics(field)
    else:
        args["statistics"] = None

    field = data.get("reputation", None)
    if field is not None:
        args["reputation"] = unmarshal_DomainReputation(field)
    else:
        args["reputation"] = None

    field = data.get("records", None)
    if field is not None:
        args["records"] = unmarshal_DomainRecords(field)
    else:
        args["records"] = None

    return Domain(**args)


def unmarshal_OfferSubscription(data: Any) -> OfferSubscription:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'OfferSubscription' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("organization_id", None)
    if field is not None:
        args["organization_id"] = field

    field = data.get("project_id", None)
    if field is not None:
        args["project_id"] = field

    field = data.get("offer_name", None)
    if field is not None:
        args["offer_name"] = field

    field = data.get("sla", None)
    if field is not None:
        args["sla"] = field

    field = data.get("max_domains", None)
    if field is not None:
        args["max_domains"] = field

    field = data.get("max_dedicated_ips", None)
    if field is not None:
        args["max_dedicated_ips"] = field

    field = data.get("max_webhooks_per_domain", None)
    if field is not None:
        args["max_webhooks_per_domain"] = field

    field = data.get("max_custom_blocklists_per_domain", None)
    if field is not None:
        args["max_custom_blocklists_per_domain"] = field

    field = data.get("included_monthly_emails", None)
    if field is not None:
        args["included_monthly_emails"] = field

    field = data.get("subscribed_at", None)
    if field is not None:
        args["subscribed_at"] = (
            parser.isoparse(field) if isinstance(field, str) else field
        )
    else:
        args["subscribed_at"] = None

    field = data.get("cancellation_available_at", None)
    if field is not None:
        args["cancellation_available_at"] = (
            parser.isoparse(field) if isinstance(field, str) else field
        )
    else:
        args["cancellation_available_at"] = None

    return OfferSubscription(**args)


def unmarshal_Webhook(data: Any) -> Webhook:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'Webhook' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("id", None)
    if field is not None:
        args["id"] = field

    field = data.get("domain_id", None)
    if field is not None:
        args["domain_id"] = field

    field = data.get("organization_id", None)
    if field is not None:
        args["organization_id"] = field

    field = data.get("project_id", None)
    if field is not None:
        args["project_id"] = field

    field = data.get("name", None)
    if field is not None:
        args["name"] = field

    field = data.get("event_types", None)
    if field is not None:
        args["event_types"] = (
            [WebhookEventType(v) for v in field] if field is not None else None
        )

    field = data.get("sns_arn", None)
    if field is not None:
        args["sns_arn"] = field

    field = data.get("created_at", None)
    if field is not None:
        args["created_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["created_at"] = None

    field = data.get("updated_at", None)
    if field is not None:
        args["updated_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["updated_at"] = None

    return Webhook(**args)


def unmarshal_Blocklist(data: Any) -> Blocklist:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'Blocklist' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("id", None)
    if field is not None:
        args["id"] = field

    field = data.get("domain_id", None)
    if field is not None:
        args["domain_id"] = field

    field = data.get("email", None)
    if field is not None:
        args["email"] = field

    field = data.get("type", None)
    if field is not None:
        args["type_"] = field

    field = data.get("reason", None)
    if field is not None:
        args["reason"] = field

    field = data.get("custom", None)
    if field is not None:
        args["custom"] = field

    field = data.get("created_at", None)
    if field is not None:
        args["created_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["created_at"] = None

    field = data.get("updated_at", None)
    if field is not None:
        args["updated_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["updated_at"] = None

    field = data.get("ends_at", None)
    if field is not None:
        args["ends_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["ends_at"] = None

    return Blocklist(**args)


def unmarshal_BulkCreateBlocklistsResponse(data: Any) -> BulkCreateBlocklistsResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'BulkCreateBlocklistsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("blocklists", None)
    if field is not None:
        args["blocklists"] = (
            [unmarshal_Blocklist(v) for v in field] if field is not None else None
        )

    return BulkCreateBlocklistsResponse(**args)


def unmarshal_CreateEmailResponse(data: Any) -> CreateEmailResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'CreateEmailResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("emails", None)
    if field is not None:
        args["emails"] = (
            [unmarshal_Email(v) for v in field] if field is not None else None
        )

    return CreateEmailResponse(**args)


def unmarshal_DomainLastStatusAutoconfigState(
    data: Any,
) -> DomainLastStatusAutoconfigState:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'DomainLastStatusAutoconfigState' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("enabled", None)
    if field is not None:
        args["enabled"] = field

    field = data.get("autoconfigurable", None)
    if field is not None:
        args["autoconfigurable"] = field

    field = data.get("reason", None)
    if field is not None:
        args["reason"] = field
    else:
        args["reason"] = None

    return DomainLastStatusAutoconfigState(**args)


def unmarshal_DomainLastStatusDkimRecord(data: Any) -> DomainLastStatusDkimRecord:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'DomainLastStatusDkimRecord' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("status", None)
    if field is not None:
        args["status"] = field

    field = data.get("last_valid_at", None)
    if field is not None:
        args["last_valid_at"] = (
            parser.isoparse(field) if isinstance(field, str) else field
        )
    else:
        args["last_valid_at"] = None

    field = data.get("error", None)
    if field is not None:
        args["error"] = field
    else:
        args["error"] = None

    return DomainLastStatusDkimRecord(**args)


def unmarshal_DomainLastStatusDmarcRecord(data: Any) -> DomainLastStatusDmarcRecord:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'DomainLastStatusDmarcRecord' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("status", None)
    if field is not None:
        args["status"] = field

    field = data.get("last_valid_at", None)
    if field is not None:
        args["last_valid_at"] = (
            parser.isoparse(field) if isinstance(field, str) else field
        )
    else:
        args["last_valid_at"] = None

    field = data.get("error", None)
    if field is not None:
        args["error"] = field
    else:
        args["error"] = None

    return DomainLastStatusDmarcRecord(**args)


def unmarshal_DomainLastStatusSpfRecord(data: Any) -> DomainLastStatusSpfRecord:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'DomainLastStatusSpfRecord' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("status", None)
    if field is not None:
        args["status"] = field

    field = data.get("last_valid_at", None)
    if field is not None:
        args["last_valid_at"] = (
            parser.isoparse(field) if isinstance(field, str) else field
        )
    else:
        args["last_valid_at"] = None

    field = data.get("error", None)
    if field is not None:
        args["error"] = field
    else:
        args["error"] = None

    return DomainLastStatusSpfRecord(**args)


def unmarshal_DomainLastStatus(data: Any) -> DomainLastStatus:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'DomainLastStatus' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("domain_id", None)
    if field is not None:
        args["domain_id"] = field

    field = data.get("domain_name", None)
    if field is not None:
        args["domain_name"] = field

    field = data.get("spf_record", None)
    if field is not None:
        args["spf_record"] = unmarshal_DomainLastStatusSpfRecord(field)
    else:
        args["spf_record"] = None

    field = data.get("dkim_record", None)
    if field is not None:
        args["dkim_record"] = unmarshal_DomainLastStatusDkimRecord(field)
    else:
        args["dkim_record"] = None

    field = data.get("dmarc_record", None)
    if field is not None:
        args["dmarc_record"] = unmarshal_DomainLastStatusDmarcRecord(field)
    else:
        args["dmarc_record"] = None

    field = data.get("autoconfig_state", None)
    if field is not None:
        args["autoconfig_state"] = unmarshal_DomainLastStatusAutoconfigState(field)
    else:
        args["autoconfig_state"] = None

    return DomainLastStatus(**args)


def unmarshal_ListBlocklistsResponse(data: Any) -> ListBlocklistsResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ListBlocklistsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("total_count", None)
    if field is not None:
        args["total_count"] = field

    field = data.get("blocklists", None)
    if field is not None:
        args["blocklists"] = (
            [unmarshal_Blocklist(v) for v in field] if field is not None else None
        )

    return ListBlocklistsResponse(**args)


def unmarshal_ListDomainsResponse(data: Any) -> ListDomainsResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ListDomainsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("total_count", None)
    if field is not None:
        args["total_count"] = field

    field = data.get("domains", None)
    if field is not None:
        args["domains"] = (
            [unmarshal_Domain(v) for v in field] if field is not None else None
        )

    return ListDomainsResponse(**args)


def unmarshal_ListEmailsResponse(data: Any) -> ListEmailsResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ListEmailsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("total_count", None)
    if field is not None:
        args["total_count"] = field

    field = data.get("emails", None)
    if field is not None:
        args["emails"] = (
            [unmarshal_Email(v) for v in field] if field is not None else None
        )

    return ListEmailsResponse(**args)


def unmarshal_ListOfferSubscriptionsResponse(
    data: Any,
) -> ListOfferSubscriptionsResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ListOfferSubscriptionsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("total_count", None)
    if field is not None:
        args["total_count"] = field

    field = data.get("offer_subscriptions", None)
    if field is not None:
        args["offer_subscriptions"] = (
            [unmarshal_OfferSubscription(v) for v in field]
            if field is not None
            else None
        )

    return ListOfferSubscriptionsResponse(**args)


def unmarshal_Offer(data: Any) -> Offer:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'Offer' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("name", None)
    if field is not None:
        args["name"] = field

    field = data.get("sla", None)
    if field is not None:
        args["sla"] = field

    field = data.get("max_domains", None)
    if field is not None:
        args["max_domains"] = field

    field = data.get("max_dedicated_ips", None)
    if field is not None:
        args["max_dedicated_ips"] = field

    field = data.get("included_monthly_emails", None)
    if field is not None:
        args["included_monthly_emails"] = field

    field = data.get("max_webhooks_per_domain", None)
    if field is not None:
        args["max_webhooks_per_domain"] = field

    field = data.get("max_custom_blocklists_per_domain", None)
    if field is not None:
        args["max_custom_blocklists_per_domain"] = field

    field = data.get("created_at", None)
    if field is not None:
        args["created_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["created_at"] = None

    field = data.get("commitment_period", None)
    if field is not None:
        args["commitment_period"] = field
    else:
        args["commitment_period"] = None

    return Offer(**args)


def unmarshal_ListOffersResponse(data: Any) -> ListOffersResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ListOffersResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("total_count", None)
    if field is not None:
        args["total_count"] = field

    field = data.get("offers", None)
    if field is not None:
        args["offers"] = (
            [unmarshal_Offer(v) for v in field] if field is not None else None
        )

    return ListOffersResponse(**args)


def unmarshal_Pool(data: Any) -> Pool:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'Pool' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("project_id", None)
    if field is not None:
        args["project_id"] = field

    field = data.get("status", None)
    if field is not None:
        args["status"] = field

    field = data.get("ips", None)
    if field is not None:
        args["ips"] = field

    field = data.get("details", None)
    if field is not None:
        args["details"] = field
    else:
        args["details"] = None

    field = data.get("zone", None)
    if field is not None:
        args["zone"] = field
    else:
        args["zone"] = None

    field = data.get("reverse", None)
    if field is not None:
        args["reverse"] = field
    else:
        args["reverse"] = None

    return Pool(**args)


def unmarshal_ListPoolsResponse(data: Any) -> ListPoolsResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ListPoolsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("total_count", None)
    if field is not None:
        args["total_count"] = field

    field = data.get("pools", None)
    if field is not None:
        args["pools"] = (
            [unmarshal_Pool(v) for v in field] if field is not None else None
        )

    return ListPoolsResponse(**args)


def unmarshal_WebhookEvent(data: Any) -> WebhookEvent:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'WebhookEvent' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("id", None)
    if field is not None:
        args["id"] = field

    field = data.get("webhook_id", None)
    if field is not None:
        args["webhook_id"] = field

    field = data.get("organization_id", None)
    if field is not None:
        args["organization_id"] = field

    field = data.get("project_id", None)
    if field is not None:
        args["project_id"] = field

    field = data.get("domain_id", None)
    if field is not None:
        args["domain_id"] = field

    field = data.get("type", None)
    if field is not None:
        args["type_"] = field

    field = data.get("status", None)
    if field is not None:
        args["status"] = field

    field = data.get("data", None)
    if field is not None:
        args["data"] = field

    field = data.get("created_at", None)
    if field is not None:
        args["created_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["created_at"] = None

    field = data.get("updated_at", None)
    if field is not None:
        args["updated_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["updated_at"] = None

    field = data.get("email_id", None)
    if field is not None:
        args["email_id"] = field
    else:
        args["email_id"] = None

    return WebhookEvent(**args)


def unmarshal_ListWebhookEventsResponse(data: Any) -> ListWebhookEventsResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ListWebhookEventsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("total_count", None)
    if field is not None:
        args["total_count"] = field

    field = data.get("webhook_events", None)
    if field is not None:
        args["webhook_events"] = (
            [unmarshal_WebhookEvent(v) for v in field] if field is not None else None
        )

    return ListWebhookEventsResponse(**args)


def unmarshal_ListWebhooksResponse(data: Any) -> ListWebhooksResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ListWebhooksResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("total_count", None)
    if field is not None:
        args["total_count"] = field

    field = data.get("webhooks", None)
    if field is not None:
        args["webhooks"] = (
            [unmarshal_Webhook(v) for v in field] if field is not None else None
        )

    return ListWebhooksResponse(**args)


def unmarshal_ProjectConsumption(data: Any) -> ProjectConsumption:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ProjectConsumption' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("project_id", None)
    if field is not None:
        args["project_id"] = field

    field = data.get("domains_count", None)
    if field is not None:
        args["domains_count"] = field

    field = data.get("dedicated_ips_count", None)
    if field is not None:
        args["dedicated_ips_count"] = field

    field = data.get("monthly_emails_count", None)
    if field is not None:
        args["monthly_emails_count"] = field

    field = data.get("webhooks_count", None)
    if field is not None:
        args["webhooks_count"] = field

    field = data.get("custom_blocklists_count", None)
    if field is not None:
        args["custom_blocklists_count"] = field

    return ProjectConsumption(**args)


def unmarshal_ProjectSettingsPeriodicReport(data: Any) -> ProjectSettingsPeriodicReport:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ProjectSettingsPeriodicReport' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("enabled", None)
    if field is not None:
        args["enabled"] = field

    field = data.get("frequency", None)
    if field is not None:
        args["frequency"] = field

    field = data.get("sending_hour", None)
    if field is not None:
        args["sending_hour"] = field

    field = data.get("sending_day", None)
    if field is not None:
        args["sending_day"] = field

    return ProjectSettingsPeriodicReport(**args)


def unmarshal_ProjectSettings(data: Any) -> ProjectSettings:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ProjectSettings' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("periodic_report", None)
    if field is not None:
        args["periodic_report"] = unmarshal_ProjectSettingsPeriodicReport(field)
    else:
        args["periodic_report"] = None

    return ProjectSettings(**args)


def unmarshal_Statistics(data: Any) -> Statistics:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'Statistics' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("total_count", None)
    if field is not None:
        args["total_count"] = field

    field = data.get("new_count", None)
    if field is not None:
        args["new_count"] = field

    field = data.get("sending_count", None)
    if field is not None:
        args["sending_count"] = field

    field = data.get("sent_count", None)
    if field is not None:
        args["sent_count"] = field

    field = data.get("failed_count", None)
    if field is not None:
        args["failed_count"] = field

    field = data.get("canceled_count", None)
    if field is not None:
        args["canceled_count"] = field

    return Statistics(**args)


def marshal_BulkCreateBlocklistsRequest(
    request: BulkCreateBlocklistsRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.domain_id is not None:
        output["domain_id"] = request.domain_id

    if request.emails is not None:
        output["emails"] = request.emails

    if request.type_ is not None:
        output["type"] = str(request.type_)

    if request.reason is not None:
        output["reason"] = request.reason

    return output


def marshal_CreateDomainRequest(
    request: CreateDomainRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.domain_name is not None:
        output["domain_name"] = request.domain_name

    if request.autoconfig is not None:
        output["autoconfig"] = request.autoconfig

    if request.project_id is not None:
        output["project_id"] = request.project_id or defaults.default_project_id

    if request.accept_tos is not None:
        output["accept_tos"] = request.accept_tos

    return output


def marshal_CreateEmailRequestAddress(
    request: CreateEmailRequestAddress,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.email is not None:
        output["email"] = request.email

    if request.name is not None:
        output["name"] = request.name

    return output


def marshal_CreateEmailRequestAttachment(
    request: CreateEmailRequestAttachment,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.name is not None:
        output["name"] = request.name

    if request.type_ is not None:
        output["type"] = request.type_

    if request.content is not None:
        output["content"] = request.content

    return output


def marshal_CreateEmailRequestHeader(
    request: CreateEmailRequestHeader,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.key is not None:
        output["key"] = request.key

    if request.value is not None:
        output["value"] = request.value

    return output


def marshal_CreateEmailRequest(
    request: CreateEmailRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.from_ is not None:
        output["from"] = marshal_CreateEmailRequestAddress(request.from_, defaults)

    if request.subject is not None:
        output["subject"] = request.subject

    if request.text is not None:
        output["text"] = request.text

    if request.html is not None:
        output["html"] = request.html

    if request.to is not None:
        output["to"] = [
            marshal_CreateEmailRequestAddress(item, defaults) for item in request.to
        ]

    if request.cc is not None:
        output["cc"] = [
            marshal_CreateEmailRequestAddress(item, defaults) for item in request.cc
        ]

    if request.bcc is not None:
        output["bcc"] = [
            marshal_CreateEmailRequestAddress(item, defaults) for item in request.bcc
        ]

    if request.project_id is not None:
        output["project_id"] = request.project_id or defaults.default_project_id

    if request.attachments is not None:
        output["attachments"] = [
            marshal_CreateEmailRequestAttachment(item, defaults)
            for item in request.attachments
        ]

    if request.send_before is not None:
        output["send_before"] = request.send_before.isoformat()

    if request.additional_headers is not None:
        output["additional_headers"] = [
            marshal_CreateEmailRequestHeader(item, defaults)
            for item in request.additional_headers
        ]

    return output


def marshal_CreateWebhookRequest(
    request: CreateWebhookRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.domain_id is not None:
        output["domain_id"] = request.domain_id

    if request.name is not None:
        output["name"] = request.name

    if request.sns_arn is not None:
        output["sns_arn"] = request.sns_arn

    if request.project_id is not None:
        output["project_id"] = request.project_id or defaults.default_project_id

    if request.event_types is not None:
        output["event_types"] = [str(item) for item in request.event_types]

    return output


def marshal_UpdateDomainRequest(
    request: UpdateDomainRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.autoconfig is not None:
        output["autoconfig"] = request.autoconfig

    return output


def marshal_UpdateOfferSubscriptionRequest(
    request: UpdateOfferSubscriptionRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.project_id is not None:
        output["project_id"] = request.project_id or defaults.default_project_id

    if request.name is not None:
        output["name"] = str(request.name)

    return output


def marshal_UpdateProjectSettingsRequestUpdatePeriodicReport(
    request: UpdateProjectSettingsRequestUpdatePeriodicReport,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.enabled is not None:
        output["enabled"] = request.enabled

    if request.frequency is not None:
        output["frequency"] = str(request.frequency)

    if request.sending_hour is not None:
        output["sending_hour"] = request.sending_hour

    if request.sending_day is not None:
        output["sending_day"] = request.sending_day

    return output


def marshal_UpdateProjectSettingsRequest(
    request: UpdateProjectSettingsRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.periodic_report is not None:
        output["periodic_report"] = (
            marshal_UpdateProjectSettingsRequestUpdatePeriodicReport(
                request.periodic_report, defaults
            )
        )

    return output


def marshal_UpdateWebhookRequest(
    request: UpdateWebhookRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.name is not None:
        output["name"] = request.name

    if request.event_types is not None:
        output["event_types"] = [str(item) for item in request.event_types]

    if request.sns_arn is not None:
        output["sns_arn"] = request.sns_arn

    return output



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from .types import InstanceStatus
from .content import INSTANCE_TRANSIENT_STATUSES
from .types import ListInstancesRequestOrderBy
from .types import ListSnapshotsRequestOrderBy
from .types import ListUsersRequestOrderBy
from .types import NodeTypeStock
from .types import SettingPropertyType
from .types import SnapshotStatus
from .content import SNAPSHOT_TRANSIENT_STATUSES
from .types import UserRoleRole
from .types import VolumeType
from .types import EndpointPrivateNetworkDetails
from .types import EndpointPublicDetails
from .types import EndpointSpecPrivateNetworkDetails
from .types import EndpointSpecPublicDetails
from .types import Endpoint
from .types import InstanceSetting
from .types import Volume
from .types import NodeTypeVolumeType
from .types import SnapshotVolumeType
from .types import UserRole
from .types import Setting
from .types import EndpointSpec
from .types import CreateInstanceRequestVolumeDetails
from .types import Instance
from .types import NodeType
from .types import Snapshot
from .types import User
from .types import Version
from .types import RestoreSnapshotRequestVolumeDetails
from .types import CreateEndpointRequest
from .types import CreateInstanceRequest
from .types import CreateSnapshotRequest
from .types import CreateUserRequest
from .types import DeleteEndpointRequest
from .types import DeleteInstanceRequest
from .types import DeleteSnapshotRequest
from .types import DeleteUserRequest
from .types import GetInstanceCertificateRequest
from .types import GetInstanceRequest
from .types import GetSnapshotRequest
from .types import ListInstancesRequest
from .types import ListInstancesResponse
from .types import ListNodeTypesRequest
from .types import ListNodeTypesResponse
from .types import ListSnapshotsRequest
from .types import ListSnapshotsResponse
from .types import ListUsersRequest
from .types import ListUsersResponse
from .types import ListVersionsRequest
from .types import ListVersionsResponse
from .types import RestoreSnapshotRequest
from .types import SetUserRoleRequest
from .types import UpdateInstanceRequest
from .types import UpdateSnapshotRequest
from .types import UpdateUserRequest
from .types import UpgradeInstanceRequest
from .api import MongodbV1Alpha1API

__all__ = [
    "InstanceStatus",
    "INSTANCE_TRANSIENT_STATUSES",
    "ListInstancesRequestOrderBy",
    "ListSnapshotsRequestOrderBy",
    "ListUsersRequestOrderBy",
    "NodeTypeStock",
    "SettingPropertyType",
    "SnapshotStatus",
    "SNAPSHOT_TRANSIENT_STATUSES",
    "UserRoleRole",
    "VolumeType",
    "EndpointPrivateNetworkDetails",
    "EndpointPublicDetails",
    "EndpointSpecPrivateNetworkDetails",
    "EndpointSpecPublicDetails",
    "Endpoint",
    "InstanceSetting",
    "Volume",
    "NodeTypeVolumeType",
    "SnapshotVolumeType",
    "UserRole",
    "Setting",
    "EndpointSpec",
    "CreateInstanceRequestVolumeDetails",
    "Instance",
    "NodeType",
    "Snapshot",
    "User",
    "Version",
    "RestoreSnapshotRequestVolumeDetails",
    "CreateEndpointRequest",
    "CreateInstanceRequest",
    "CreateSnapshotRequest",
    "CreateUserRequest",
    "DeleteEndpointRequest",
    "DeleteInstanceRequest",
    "DeleteSnapshotRequest",
    "DeleteUserRequest",
    "GetInstanceCertificateRequest",
    "GetInstanceRequest",
    "GetSnapshotRequest",
    "ListInstancesRequest",
    "ListInstancesResponse",
    "ListNodeTypesRequest",
    "ListNodeTypesResponse",
    "ListSnapshotsRequest",
    "ListSnapshotsResponse",
    "ListUsersRequest",
    "ListUsersResponse",
    "ListVersionsRequest",
    "ListVersionsResponse",
    "RestoreSnapshotRequest",
    "SetUserRoleRequest",
    "UpdateInstanceRequest",
    "UpdateSnapshotRequest",
    "UpdateUserRequest",
    "UpgradeInstanceRequest",
    "MongodbV1Alpha1API",
]



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from .types import ListImagesRequestOrderBy
from .types import ListLocalImagesRequestOrderBy
from .types import ListVersionsRequestOrderBy
from .types import LocalImageType
from .types import Category
from .types import Image
from .types import LocalImage
from .types import Version
from .types import GetCategoryRequest
from .types import GetImageRequest
from .types import GetLocalImageRequest
from .types import GetVersionRequest
from .types import ListCategoriesRequest
from .types import ListCategoriesResponse
from .types import ListImagesRequest
from .types import ListImagesResponse
from .types import ListLocalImagesRequest
from .types import ListLocalImagesResponse
from .types import ListVersionsRequest
from .types import ListVersionsResponse
from .api import MarketplaceV2API

__all__ = [
    "ListImagesRequestOrderBy",
    "ListLocalImagesRequestOrderBy",
    "ListVersionsRequestOrderBy",
    "LocalImageType",
    "Category",
    "Image",
    "LocalImage",
    "Version",
    "GetCategoryRequest",
    "GetImageRequest",
    "GetLocalImageRequest",
    "GetVersionRequest",
    "ListCategoriesRequest",
    "ListCategoriesResponse",
    "ListImagesRequest",
    "ListImagesResponse",
    "ListLocalImagesRequest",
    "ListLocalImagesResponse",
    "ListVersionsRequest",
    "ListVersionsResponse",
    "MarketplaceV2API",
]



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from .types import BearerType
from .types import GracePeriodType
from .types import ListAPIKeysRequestOrderBy
from .types import ListApplicationsRequestOrderBy
from .types import ListGroupsRequestOrderBy
from .types import ListJWTsRequestOrderBy
from .types import ListLogsRequestOrderBy
from .types import ListPermissionSetsRequestOrderBy
from .types import ListPoliciesRequestOrderBy
from .types import ListQuotaRequestOrderBy
from .types import ListSSHKeysRequestOrderBy
from .types import ListUsersRequestOrderBy
from .types import LocalityType
from .types import LogAction
from .types import LogResourceType
from .types import PermissionSetScopeType
from .types import UserStatus
from .types import UserType
from .types import ConnectionConnectedOrganization
from .types import ConnectionConnectedUser
from .types import QuotumLimit
from .types import JWT
from .types import RuleSpecs
from .types import CreateUserRequestMember
from .types import Connection
from .types import APIKey
from .types import Application
from .types import GracePeriod
from .types import Group
from .types import Log
from .types import PermissionSet
from .types import Policy
from .types import Quotum
from .types import Rule
from .types import SSHKey
from .types import User
from .types import AddGroupMemberRequest
from .types import AddGroupMembersRequest
from .types import ClonePolicyRequest
from .types import CreateAPIKeyRequest
from .types import CreateApplicationRequest
from .types import CreateGroupRequest
from .types import CreateJWTRequest
from .types import CreatePolicyRequest
from .types import CreateSSHKeyRequest
from .types import CreateUserMFAOTPRequest
from .types import CreateUserRequest
from .types import DeleteAPIKeyRequest
from .types import DeleteApplicationRequest
from .types import DeleteGroupRequest
from .types import DeleteJWTRequest
from .types import DeletePolicyRequest
from .types import DeleteSSHKeyRequest
from .types import DeleteUserMFAOTPRequest
from .types import DeleteUserRequest
from .types import EncodedJWT
from .types import GetAPIKeyRequest
from .types import GetApplicationRequest
from .types import GetGroupRequest
from .types import GetJWTRequest
from .types import GetLogRequest
from .types import GetOrganizationSecuritySettingsRequest
from .types import GetPolicyRequest
from .types import GetQuotumRequest
from .types import GetSSHKeyRequest
from .types import GetUserConnectionsRequest
from .types import GetUserConnectionsResponse
from .types import GetUserRequest
from .types import ListAPIKeysRequest
from .types import ListAPIKeysResponse
from .types import ListApplicationsRequest
from .types import ListApplicationsResponse
from .types import ListGracePeriodsRequest
from .types import ListGracePeriodsResponse
from .types import ListGroupsRequest
from .types import ListGroupsResponse
from .types import ListJWTsRequest
from .types import ListJWTsResponse
from .types import ListLogsRequest
from .types import ListLogsResponse
from .types import ListPermissionSetsRequest
from .types import ListPermissionSetsResponse
from .types import ListPoliciesRequest
from .types import ListPoliciesResponse
from .types import ListQuotaRequest
from .types import ListQuotaResponse
from .types import ListRulesRequest
from .types import ListRulesResponse
from .types import ListSSHKeysRequest
from .types import ListSSHKeysResponse
from .types import ListUsersRequest
from .types import ListUsersResponse
from .types import LockUserRequest
from .types import MFAOTP
from .types import OrganizationSecuritySettings
from .types import RemoveGroupMemberRequest
from .types import SetGroupMembersRequest
from .types import SetRulesRequest
from .types import SetRulesResponse
from .types import UnlockUserRequest
from .types import UpdateAPIKeyRequest
from .types import UpdateApplicationRequest
from .types import UpdateGroupRequest
from .types import UpdateOrganizationSecuritySettingsRequest
from .types import UpdatePolicyRequest
from .types import UpdateSSHKeyRequest
from .types import UpdateUserPasswordRequest
from .types import UpdateUserRequest
from .types import UpdateUserUsernameRequest
from .types import ValidateUserMFAOTPRequest
from .types import ValidateUserMFAOTPResponse
from .api import IamV1Alpha1API

__all__ = [
    "BearerType",
    "GracePeriodType",
    "ListAPIKeysRequestOrderBy",
    "ListApplicationsRequestOrderBy",
    "ListGroupsRequestOrderBy",
    "ListJWTsRequestOrderBy",
    "ListLogsRequestOrderBy",
    "ListPermissionSetsRequestOrderBy",
    "ListPoliciesRequestOrderBy",
    "ListQuotaRequestOrderBy",
    "ListSSHKeysRequestOrderBy",
    "ListUsersRequestOrderBy",
    "LocalityType",
    "LogAction",
    "LogResourceType",
    "PermissionSetScopeType",
    "UserStatus",
    "UserType",
    "ConnectionConnectedOrganization",
    "ConnectionConnectedUser",
    "QuotumLimit",
    "JWT",
    "RuleSpecs",
    "CreateUserRequestMember",
    "Connection",
    "APIKey",
    "Application",
    "GracePeriod",
    "Group",
    "Log",
    "PermissionSet",
    "Policy",
    "Quotum",
    "Rule",
    "SSHKey",
    "User",
    "AddGroupMemberRequest",
    "AddGroupMembersRequest",
    "ClonePolicyRequest",
    "CreateAPIKeyRequest",
    "CreateApplicationRequest",
    "CreateGroupRequest",
    "CreateJWTRequest",
    "CreatePolicyRequest",
    "CreateSSHKeyRequest",
    "CreateUserMFAOTPRequest",
    "CreateUserRequest",
    "DeleteAPIKeyRequest",
    "DeleteApplicationRequest",
    "DeleteGroupRequest",
    "DeleteJWTRequest",
    "DeletePolicyRequest",
    "DeleteSSHKeyRequest",
    "DeleteUserMFAOTPRequest",
    "DeleteUserRequest",
    "EncodedJWT",
    "GetAPIKeyRequest",
    "GetApplicationRequest",
    "GetGroupRequest",
    "GetJWTRequest",
    "GetLogRequest",
    "GetOrganizationSecuritySettingsRequest",
    "GetPolicyRequest",
    "GetQuotumRequest",
    "GetSSHKeyRequest",
    "GetUserConnectionsRequest",
    "GetUserConnectionsResponse",
    "GetUserRequest",
    "ListAPIKeysRequest",
    "ListAPIKeysResponse",
    "ListApplicationsRequest",
    "ListApplicationsResponse",
    "ListGracePeriodsRequest",
    "ListGracePeriodsResponse",
    "ListGroupsRequest",
    "ListGroupsResponse",
    "ListJWTsRequest",
    "ListJWTsResponse",
    "ListLogsRequest",
    "ListLogsResponse",
    "ListPermissionSetsRequest",
    "ListPermissionSetsResponse",
    "ListPoliciesRequest",
    "ListPoliciesResponse",
    "ListQuotaRequest",
    "ListQuotaResponse",
    "ListRulesRequest",
    "ListRulesResponse",
    "ListSSHKeysRequest",
    "ListSSHKeysResponse",
    "ListUsersRequest",
    "ListUsersResponse",
    "LockUserRequest",
    "MFAOTP",
    "OrganizationSecuritySettings",
    "RemoveGroupMemberRequest",
    "SetGroupMembersRequest",
    "SetRulesRequest",
    "SetRulesResponse",
    "UnlockUserRequest",
    "UpdateAPIKeyRequest",
    "UpdateApplicationRequest",
    "UpdateGroupRequest",
    "UpdateOrganizationSecuritySettingsRequest",
    "UpdatePolicyRequest",
    "UpdateSSHKeyRequest",
    "UpdateUserPasswordRequest",
    "UpdateUserRequest",
    "UpdateUserUsernameRequest",
    "ValidateUserMFAOTPRequest",
    "ValidateUserMFAOTPResponse",
    "IamV1Alpha1API",
]



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from .types import DiscountDiscountMode
from .types import DiscountFilterType
from .types import DownloadInvoiceRequestFileType
from .types import ExportInvoicesRequestFileType
from .types import ExportInvoicesRequestOrderBy
from .types import InvoiceType
from .types import ListConsumptionsRequestOrderBy
from .types import ListDiscountsRequestOrderBy
from .types import ListInvoicesRequestOrderBy
from .types import ListTaxesRequestOrderBy
from .types import DiscountCoupon
from .types import DiscountFilter
from .types import ListConsumptionsResponseConsumption
from .types import Discount
from .types import Invoice
from .types import ListTaxesResponseTax
from .types import DownloadInvoiceRequest
from .types import ExportInvoicesRequest
from .types import GetInvoiceRequest
from .types import ListConsumptionsRequest
from .types import ListConsumptionsResponse
from .types import ListDiscountsRequest
from .types import ListDiscountsResponse
from .types import ListInvoicesRequest
from .types import ListInvoicesResponse
from .types import ListTaxesRequest
from .types import ListTaxesResponse
from .api import BillingV2Beta1API

__all__ = [
    "DiscountDiscountMode",
    "DiscountFilterType",
    "DownloadInvoiceRequestFileType",
    "ExportInvoicesRequestFileType",
    "ExportInvoicesRequestOrderBy",
    "InvoiceType",
    "ListConsumptionsRequestOrderBy",
    "ListDiscountsRequestOrderBy",
    "ListInvoicesRequestOrderBy",
    "ListTaxesRequestOrderBy",
    "DiscountCoupon",
    "DiscountFilter",
    "ListConsumptionsResponseConsumption",
    "Discount",
    "Invoice",
    "ListTaxesResponseTax",
    "DownloadInvoiceRequest",
    "ExportInvoicesRequest",
    "GetInvoiceRequest",
    "ListConsumptionsRequest",
    "ListConsumptionsResponse",
    "ListDiscountsRequest",
    "ListDiscountsResponse",
    "ListInvoicesRequest",
    "ListInvoicesResponse",
    "ListTaxesRequest",
    "ListTaxesResponse",
    "BillingV2Beta1API",
]



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from .types import BgpStatus
from .types import LinkStatus
from .types import ListLinksRequestOrderBy
from .types import ListPartnersRequestOrderBy
from .types import ListPopsRequestOrderBy
from .types import ListRoutingPoliciesRequestOrderBy
from .types import Link
from .types import Partner
from .types import Pop
from .types import RoutingPolicy
from .types import AttachRoutingPolicyRequest
from .types import AttachVpcRequest
from .types import CreateLinkRequest
from .types import CreateRoutingPolicyRequest
from .types import DeleteLinkRequest
from .types import DeleteRoutingPolicyRequest
from .types import DetachRoutingPolicyRequest
from .types import DetachVpcRequest
from .types import DisableRoutePropagationRequest
from .types import EnableRoutePropagationRequest
from .types import GetLinkRequest
from .types import GetPartnerRequest
from .types import GetPopRequest
from .types import GetRoutingPolicyRequest
from .types import ListLinksRequest
from .types import ListLinksResponse
from .types import ListPartnersRequest
from .types import ListPartnersResponse
from .types import ListPopsRequest
from .types import ListPopsResponse
from .types import ListRoutingPoliciesRequest
from .types import ListRoutingPoliciesResponse
from .types import UpdateLinkRequest
from .types import UpdateRoutingPolicyRequest
from .api import InterlinkV1Beta1API

__all__ = [
    "BgpStatus",
    "LinkStatus",
    "ListLinksRequestOrderBy",
    "ListPartnersRequestOrderBy",
    "ListPopsRequestOrderBy",
    "ListRoutingPoliciesRequestOrderBy",
    "Link",
    "Partner",
    "Pop",
    "RoutingPolicy",
    "AttachRoutingPolicyRequest",
    "AttachVpcRequest",
    "CreateLinkRequest",
    "CreateRoutingPolicyRequest",
    "DeleteLinkRequest",
    "DeleteRoutingPolicyRequest",
    "DetachRoutingPolicyRequest",
    "DetachVpcRequest",
    "DisableRoutePropagationRequest",
    "EnableRoutePropagationRequest",
    "GetLinkRequest",
    "GetPartnerRequest",
    "GetPopRequest",
    "GetRoutingPolicyRequest",
    "ListLinksRequest",
    "ListLinksResponse",
    "ListPartnersRequest",
    "ListPartnersResponse",
    "ListPopsRequest",
    "ListPopsResponse",
    "ListRoutingPoliciesRequest",
    "ListRoutingPoliciesResponse",
    "UpdateLinkRequest",
    "UpdateRoutingPolicyRequest",
    "InterlinkV1Beta1API",
]



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from typing import List

from .types import (
    ServerPrivateNetworkStatus,
)

SERVER_PRIVATE_NETWORK_TRANSIENT_STATUSES: List[ServerPrivateNetworkStatus] = [
    ServerPrivateNetworkStatus.ATTACHING,
    ServerPrivateNetworkStatus.DETACHING,
]
"""
Lists transient statutes of the enum :class:`ServerPrivateNetworkStatus <ServerPrivateNetworkStatus>`.
"""



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from __future__ import annotations

from enum import Enum

from scaleway_core.utils import (
    StrEnumMeta,
)


class LanguageCode(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN_LANGUAGE_CODE = "unknown_language_code"
    EN_US = "en_us"
    FR_FR = "fr_fr"
    DE_DE = "de_de"

    def __str__(self) -> str:
        return str(self.value)



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.

from typing import List, Optional

from scaleway_core.api import API
from scaleway_core.bridge import (
    Region as ScwRegion,
)
from scaleway_core.utils import (
    OneOfPossibility,
    resolve_one_of,
    validate_path_param,
    fetch_all_pages,
)
from .types import (
    ListIPsRequestOrderBy,
    ResourceType,
    AttachIPRequest,
    BookIPRequest,
    CustomResource,
    DetachIPRequest,
    IP,
    ListIPsResponse,
    MoveIPRequest,
    ReleaseIPSetRequest,
    Reverse,
    Source,
    UpdateIPRequest,
)
from .marshalling import (
    unmarshal_IP,
    unmarshal_ListIPsResponse,
    marshal_AttachIPRequest,
    marshal_BookIPRequest,
    marshal_DetachIPRequest,
    marshal_MoveIPRequest,
    marshal_ReleaseIPSetRequest,
    marshal_UpdateIPRequest,
)


class IpamV1API(API):
    """
    This API allows you to manage your Scaleway IP addresses with our IP Address Management tool.
    """

    def book_ip(
        self,
        *,
        source: Source,
        is_ipv6: bool,
        region: Optional[ScwRegion] = None,
        project_id: Optional[str] = None,
        address: Optional[str] = None,
        tags: Optional[List[str]] = None,
        resource: Optional[CustomResource] = None,
    ) -> IP:
        """
        Reserve a new IP.
        Reserve a new IP from the specified source. Currently IPs can only be reserved from a Private Network.
        :param source: Source in which to reserve the IP. Not all sources are available for reservation.
        :param is_ipv6: Request an IPv6 instead of an IPv4.
        :param region: Region to target. If none is passed will use default region from the config.
        :param project_id: When creating an IP in a Private Network, the Project must match the Private Network's Project.
        :param address: The requested address should not include the subnet mask (/suffix). Note that only the Private Network source allows you to pick a specific IP. If the requested IP is already reserved, then the call will fail.
        :param tags: Tags for the IP.
        :param resource: Custom resource to attach to the IP being reserved. An example of a custom resource is a virtual machine hosted on an Elastic Metal server. Do not use this for attaching IP addresses to standard Scaleway resources, as it will fail - instead, see the relevant product API for an equivalent method.
        :return: :class:`IP <IP>`

        Usage:
        ::

            result = api.book_ip(
                source=Source(),
                is_ipv6=False,
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )

        res = self._request(
            "POST",
            f"/ipam/v1/regions/{param_region}/ips",
            body=marshal_BookIPRequest(
                BookIPRequest(
                    source=source,
                    is_ipv6=is_ipv6,
                    region=region,
                    project_id=project_id,
                    address=address,
                    tags=tags,
                    resource=resource,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_IP(res.json())

    def release_ip(
        self,
        *,
        ip_id: str,
        region: Optional[ScwRegion] = None,
    ) -> None:
        """
        Release an IP.
        Release an IP not currently attached to a resource, and returns it to the available IP pool.
        :param ip_id: IP ID.
        :param region: Region to target. If none is passed will use default region from the config.

        Usage:
        ::

            result = api.release_ip(
                ip_id="example",
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_ip_id = validate_path_param("ip_id", ip_id)

        res = self._request(
            "DELETE",
            f"/ipam/v1/regions/{param_region}/ips/{param_ip_id}",
            body={},
        )

        self._throw_on_error(res)

    def release_ip_set(
        self,
        *,
        region: Optional[ScwRegion] = None,
        ip_ids: Optional[List[str]] = None,
    ) -> None:
        """
        :param region: Region to target. If none is passed will use default region from the config.
        :param ip_ids:

        Usage:
        ::

            result = api.release_ip_set()
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )

        res = self._request(
            "POST",
            f"/ipam/v1/regions/{param_region}/ip-sets/release",
            body=marshal_ReleaseIPSetRequest(
                ReleaseIPSetRequest(
                    region=region,
                    ip_ids=ip_ids,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)

    def get_ip(
        self,
        *,
        ip_id: str,
        region: Optional[ScwRegion] = None,
    ) -> IP:
        """
        Get an IP.
        Retrieve details of an existing IP, specified by its IP ID.
        :param ip_id: IP ID.
        :param region: Region to target. If none is passed will use default region from the config.
        :return: :class:`IP <IP>`

        Usage:
        ::

            result = api.get_ip(
                ip_id="example",
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_ip_id = validate_path_param("ip_id", ip_id)

        res = self._request(
            "GET",
            f"/ipam/v1/regions/{param_region}/ips/{param_ip_id}",
        )

        self._throw_on_error(res)
        return unmarshal_IP(res.json())

    def update_ip(
        self,
        *,
        ip_id: str,
        region: Optional[ScwRegion] = None,
        tags: Optional[List[str]] = None,
        reverses: Optional[List[Reverse]] = None,
    ) -> IP:
        """
        Update an IP.
        Update parameters including tags of the specified IP.
        :param ip_id: IP ID.
        :param region: Region to target. If none is passed will use default region from the config.
        :param tags: Tags for the IP.
        :param reverses: Array of reverse domain names associated with an IP in the subnet of the current IP.
        :return: :class:`IP <IP>`

        Usage:
        ::

            result = api.update_ip(
                ip_id="example",
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_ip_id = validate_path_param("ip_id", ip_id)

        res = self._request(
            "PATCH",
            f"/ipam/v1/regions/{param_region}/ips/{param_ip_id}",
            body=marshal_UpdateIPRequest(
                UpdateIPRequest(
                    ip_id=ip_id,
                    region=region,
                    tags=tags,
                    reverses=reverses,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_IP(res.json())

    def list_i_ps(
        self,
        *,
        region: Optional[ScwRegion] = None,
        order_by: Optional[ListIPsRequestOrderBy] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        project_id: Optional[str] = None,
        zonal: Optional[str] = None,
        private_network_id: Optional[str] = None,
        subnet_id: Optional[str] = None,
        vpc_id: Optional[str] = None,
        attached: Optional[bool] = None,
        resource_name: Optional[str] = None,
        resource_id: Optional[str] = None,
        resource_ids: Optional[List[str]] = None,
        resource_type: Optional[ResourceType] = None,
        resource_types: Optional[List[ResourceType]] = None,
        mac_address: Optional[str] = None,
        tags: Optional[List[str]] = None,
        organization_id: Optional[str] = None,
        is_ipv6: Optional[bool] = None,
        ip_ids: Optional[List[str]] = None,
    ) -> ListIPsResponse:
        """
        List existing IPs.
        List existing IPs in the specified region using various filters. For example, you can filter for IPs within a specified Private Network, or for public IPs within a specified Project. By default, the IPs returned in the list are ordered by creation date in ascending order, though this can be modified via the order_by field.
        :param region: Region to target. If none is passed will use default region from the config.
        :param order_by: Sort order of the returned IPs.
        :param page: Page number to return, from the paginated results.
        :param page_size: Maximum number of IPs to return per page.
        :param project_id: Project ID to filter for. Only IPs belonging to this Project will be returned.
        :param zonal: Zone to filter for. Only IPs that are zonal, and in this zone, will be returned.
        One-Of ('source'): at most one of 'zonal', 'private_network_id', 'subnet_id' could be set.
        :param private_network_id: Only IPs that are private, and in this Private Network, will be returned.
        One-Of ('source'): at most one of 'zonal', 'private_network_id', 'subnet_id' could be set.
        :param subnet_id: Only IPs inside this exact subnet will be returned.
        One-Of ('source'): at most one of 'zonal', 'private_network_id', 'subnet_id' could be set.
        :param vpc_id: Only IPs owned by resources in this VPC will be returned.
        :param attached: Defines whether to filter only for IPs which are attached to a resource.
        :param resource_name: Attached resource name to filter for, only IPs attached to a resource with this string within their name will be returned.
        :param resource_id: Resource ID to filter for. Only IPs attached to this resource will be returned.
        :param resource_ids: Resource IDs to filter for. Only IPs attached to at least one of these resources will be returned.
        :param resource_type: Resource type to filter for. Only IPs attached to this type of resource will be returned.
        :param resource_types: Resource types to filter for. Only IPs attached to these types of resources will be returned.
        :param mac_address: MAC address to filter for. Only IPs attached to a resource with this MAC address will be returned.
        :param tags: Tags to filter for, only IPs with one or more matching tags will be returned.
        :param organization_id: Organization ID to filter for. Only IPs belonging to this Organization will be returned.
        :param is_ipv6: Defines whether to filter only for IPv4s or IPv6s.
        :param ip_ids: IP IDs to filter for. Only IPs with these UUIDs will be returned.
        :return: :class:`ListIPsResponse <ListIPsResponse>`

        Usage:
        ::

            result = api.list_i_ps()
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )

        res = self._request(
            "GET",
            f"/ipam/v1/regions/{param_region}/ips",
            params={
                "attached": attached,
                "ip_ids": ip_ids,
                "is_ipv6": is_ipv6,
                "mac_address": mac_address,
                "order_by": order_by,
                "organization_id": organization_id
                or self.client.default_organization_id,
                "page": page,
                "page_size": page_size or self.client.default_page_size,
                "project_id": project_id or self.client.default_project_id,
                "resource_id": resource_id,
                "resource_ids": resource_ids,
                "resource_name": resource_name,
                "resource_type": resource_type,
                "resource_types": resource_types,
                "tags": tags,
                "vpc_id": vpc_id,
                **resolve_one_of(
                    [
                        OneOfPossibility("private_network_id", private_network_id),
                        OneOfPossibility("subnet_id", subnet_id),
                        OneOfPossibility("zonal", zonal),
                    ]
                ),
            },
        )

        self._throw_on_error(res)
        return unmarshal_ListIPsResponse(res.json())

    def list_i_ps_all(
        self,
        *,
        region: Optional[ScwRegion] = None,
        order_by: Optional[ListIPsRequestOrderBy] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        project_id: Optional[str] = None,
        zonal: Optional[str] = None,
        private_network_id: Optional[str] = None,
        subnet_id: Optional[str] = None,
        vpc_id: Optional[str] = None,
        attached: Optional[bool] = None,
        resource_name: Optional[str] = None,
        resource_id: Optional[str] = None,
        resource_ids: Optional[List[str]] = None,
        resource_type: Optional[ResourceType] = None,
        resource_types: Optional[List[ResourceType]] = None,
        mac_address: Optional[str] = None,
        tags: Optional[List[str]] = None,
        organization_id: Optional[str] = None,
        is_ipv6: Optional[bool] = None,
        ip_ids: Optional[List[str]] = None,
    ) -> List[IP]:
        """
        List existing IPs.
        List existing IPs in the specified region using various filters. For example, you can filter for IPs within a specified Private Network, or for public IPs within a specified Project. By default, the IPs returned in the list are ordered by creation date in ascending order, though this can be modified via the order_by field.
        :param region: Region to target. If none is passed will use default region from the config.
        :param order_by: Sort order of the returned IPs.
        :param page: Page number to return, from the paginated results.
        :param page_size: Maximum number of IPs to return per page.
        :param project_id: Project ID to filter for. Only IPs belonging to this Project will be returned.
        :param zonal: Zone to filter for. Only IPs that are zonal, and in this zone, will be returned.
        One-Of ('source'): at most one of 'zonal', 'private_network_id', 'subnet_id' could be set.
        :param private_network_id: Only IPs that are private, and in this Private Network, will be returned.
        One-Of ('source'): at most one of 'zonal', 'private_network_id', 'subnet_id' could be set.
        :param subnet_id: Only IPs inside this exact subnet will be returned.
        One-Of ('source'): at most one of 'zonal', 'private_network_id', 'subnet_id' could be set.
        :param vpc_id: Only IPs owned by resources in this VPC will be returned.
        :param attached: Defines whether to filter only for IPs which are attached to a resource.
        :param resource_name: Attached resource name to filter for, only IPs attached to a resource with this string within their name will be returned.
        :param resource_id: Resource ID to filter for. Only IPs attached to this resource will be returned.
        :param resource_ids: Resource IDs to filter for. Only IPs attached to at least one of these resources will be returned.
        :param resource_type: Resource type to filter for. Only IPs attached to this type of resource will be returned.
        :param resource_types: Resource types to filter for. Only IPs attached to these types of resources will be returned.
        :param mac_address: MAC address to filter for. Only IPs attached to a resource with this MAC address will be returned.
        :param tags: Tags to filter for, only IPs with one or more matching tags will be returned.
        :param organization_id: Organization ID to filter for. Only IPs belonging to this Organization will be returned.
        :param is_ipv6: Defines whether to filter only for IPv4s or IPv6s.
        :param ip_ids: IP IDs to filter for. Only IPs with these UUIDs will be returned.
        :return: :class:`List[IP] <List[IP]>`

        Usage:
        ::

            result = api.list_i_ps_all()
        """

        return fetch_all_pages(
            type=ListIPsResponse,
            key="ips",
            fetcher=self.list_i_ps,
            args={
                "region": region,
                "order_by": order_by,
                "page": page,
                "page_size": page_size,
                "project_id": project_id,
                "vpc_id": vpc_id,
                "attached": attached,
                "resource_name": resource_name,
                "resource_id": resource_id,
                "resource_ids": resource_ids,
                "resource_type": resource_type,
                "resource_types": resource_types,
                "mac_address": mac_address,
                "tags": tags,
                "organization_id": organization_id,
                "is_ipv6": is_ipv6,
                "ip_ids": ip_ids,
                "zonal": zonal,
                "private_network_id": private_network_id,
                "subnet_id": subnet_id,
            },
        )

    def attach_ip(
        self,
        *,
        ip_id: str,
        resource: CustomResource,
        region: Optional[ScwRegion] = None,
    ) -> IP:
        """
        Attach IP to custom resource.
        Attach an existing reserved IP from a Private Network subnet to a custom, named resource via its MAC address. An example of a custom resource is a virtual machine hosted on an Elastic Metal server. Do not use this method for attaching IP addresses to standard Scaleway resources as it will fail - see the relevant product API for an equivalent method.
        :param ip_id: IP ID.
        :param resource: Custom resource to be attached to the IP.
        :param region: Region to target. If none is passed will use default region from the config.
        :return: :class:`IP <IP>`

        Usage:
        ::

            result = api.attach_ip(
                ip_id="example",
                resource=CustomResource(),
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_ip_id = validate_path_param("ip_id", ip_id)

        res = self._request(
            "POST",
            f"/ipam/v1/regions/{param_region}/ips/{param_ip_id}/attach",
            body=marshal_AttachIPRequest(
                AttachIPRequest(
                    ip_id=ip_id,
                    resource=resource,
                    region=region,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_IP(res.json())

    def detach_ip(
        self,
        *,
        ip_id: str,
        resource: CustomResource,
        region: Optional[ScwRegion] = None,
    ) -> IP:
        """
        Detach IP from a custom resource.
        Detach a private IP from a custom resource. An example of a custom resource is a virtual machine hosted on an Elastic Metal server. Do not use this method for detaching IP addresses from standard Scaleway resources (e.g. Instances, Load Balancers) as it will fail - see the relevant product API for an equivalent method.
        :param ip_id: IP ID.
        :param resource: Custom resource currently attached to the IP.
        :param region: Region to target. If none is passed will use default region from the config.
        :return: :class:`IP <IP>`

        Usage:
        ::

            result = api.detach_ip(
                ip_id="example",
                resource=CustomResource(),
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_ip_id = validate_path_param("ip_id", ip_id)

        res = self._request(
            "POST",
            f"/ipam/v1/regions/{param_region}/ips/{param_ip_id}/detach",
            body=marshal_DetachIPRequest(
                DetachIPRequest(
                    ip_id=ip_id,
                    resource=resource,
                    region=region,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_IP(res.json())

    def move_ip(
        self,
        *,
        ip_id: str,
        from_resource: CustomResource,
        region: Optional[ScwRegion] = None,
        to_resource: Optional[CustomResource] = None,
    ) -> IP:
        """
        Move IP to a custom resource.
        Move an existing reserved private IP from one custom resource (e.g. a virtual machine hosted on an Elastic Metal server) to another custom resource. This will detach it from the first resource, and attach it to the second. Do not use this method for moving IP addresses between standard Scaleway resources (e.g. Instances, Load Balancers) as it will fail - see the relevant product API for an equivalent method.
        :param ip_id: IP ID.
        :param from_resource: Custom resource currently attached to the IP.
        :param region: Region to target. If none is passed will use default region from the config.
        :param to_resource: Custom resource to be attached to the IP.
        :return: :class:`IP <IP>`

        Usage:
        ::

            result = api.move_ip(
                ip_id="example",
                from_resource=CustomResource(),
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_ip_id = validate_path_param("ip_id", ip_id)

        res = self._request(
            "POST",
            f"/ipam/v1/regions/{param_region}/ips/{param_ip_id}/move",
            body=marshal_MoveIPRequest(
                MoveIPRequest(
                    ip_id=ip_id,
                    from_resource=from_resource,
                    region=region,
                    to_resource=to_resource,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_IP(res.json())



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from .types import GatewayNetworkStatus
from .content import GATEWAY_NETWORK_TRANSIENT_STATUSES
from .types import GatewayStatus
from .content import GATEWAY_TRANSIENT_STATUSES
from .types import ListGatewayNetworksRequestOrderBy
from .types import ListGatewaysRequestOrderBy
from .types import ListIPsRequestOrderBy
from .types import ListPatRulesRequestOrderBy
from .types import PatRuleProtocol
from .types import GatewayNetwork
from .types import IP
from .types import GatewayType
from .types import Gateway
from .types import PatRule
from .types import SetPatRulesRequestRule
from .types import AddBastionAllowedIPsRequest
from .types import AddBastionAllowedIPsResponse
from .types import CreateGatewayNetworkRequest
from .types import CreateGatewayRequest
from .types import CreateIPRequest
from .types import CreatePatRuleRequest
from .types import DeleteBastionAllowedIPsRequest
from .types import DeleteGatewayNetworkRequest
from .types import DeleteGatewayRequest
from .types import DeleteIPRequest
from .types import DeletePatRuleRequest
from .types import GetGatewayNetworkRequest
from .types import GetGatewayRequest
from .types import GetIPRequest
from .types import GetPatRuleRequest
from .types import ListGatewayNetworksRequest
from .types import ListGatewayNetworksResponse
from .types import ListGatewayTypesRequest
from .types import ListGatewayTypesResponse
from .types import ListGatewaysRequest
from .types import ListGatewaysResponse
from .types import ListIPsRequest
from .types import ListIPsResponse
from .types import ListPatRulesRequest
from .types import ListPatRulesResponse
from .types import RefreshSSHKeysRequest
from .types import SetBastionAllowedIPsRequest
from .types import SetBastionAllowedIPsResponse
from .types import SetPatRulesRequest
from .types import SetPatRulesResponse
from .types import UpdateGatewayNetworkRequest
from .types import UpdateGatewayRequest
from .types import UpdateIPRequest
from .types import UpdatePatRuleRequest
from .types import UpgradeGatewayRequest
from .api import VpcgwV2API

__all__ = [
    "GatewayNetworkStatus",
    "GATEWAY_NETWORK_TRANSIENT_STATUSES",
    "GatewayStatus",
    "GATEWAY_TRANSIENT_STATUSES",
    "ListGatewayNetworksRequestOrderBy",
    "ListGatewaysRequestOrderBy",
    "ListIPsRequestOrderBy",
    "ListPatRulesRequestOrderBy",
    "PatRuleProtocol",
    "GatewayNetwork",
    "IP",
    "GatewayType",
    "Gateway",
    "PatRule",
    "SetPatRulesRequestRule",
    "AddBastionAllowedIPsRequest",
    "AddBastionAllowedIPsResponse",
    "CreateGatewayNetworkRequest",
    "CreateGatewayRequest",
    "CreateIPRequest",
    "CreatePatRuleRequest",
    "DeleteBastionAllowedIPsRequest",
    "DeleteGatewayNetworkRequest",
    "DeleteGatewayRequest",
    "DeleteIPRequest",
    "DeletePatRuleRequest",
    "GetGatewayNetworkRequest",
    "GetGatewayRequest",
    "GetIPRequest",
    "GetPatRuleRequest",
    "ListGatewayNetworksRequest",
    "ListGatewayNetworksResponse",
    "ListGatewayTypesRequest",
    "ListGatewayTypesResponse",
    "ListGatewaysRequest",
    "ListGatewaysResponse",
    "ListIPsRequest",
    "ListIPsResponse",
    "ListPatRulesRequest",
    "ListPatRulesResponse",
    "RefreshSSHKeysRequest",
    "SetBastionAllowedIPsRequest",
    "SetBastionAllowedIPsResponse",
    "SetPatRulesRequest",
    "SetPatRulesResponse",
    "UpdateGatewayNetworkRequest",
    "UpdateGatewayRequest",
    "UpdateIPRequest",
    "UpdatePatRuleRequest",
    "UpgradeGatewayRequest",
    "VpcgwV2API",
]



from dataclasses import dataclass
from typing import Optional, Dict

from scaleway_core.bridge import Zone as ScwZone


@dataclass
class GetServerUserDataRequest:
    server_id: str

    """
    Key defines the user data key to get
    """
    key: str

    """
    Zone of the user data to get
    """
    zone: Optional[ScwZone]


@dataclass
class GetAllServerUserDataRequest:
    server_id: str

    """
    Zone of the user data to get
    """
    zone: Optional[ScwZone]


@dataclass
class GetAllServerUserDataResponse:
    user_data: Dict[str, bytes]


@dataclass
class SetAllServerUserDataRequest:
    server_id: str

    user_data: Dict[str, bytes]

    """
    Zone of the user data to set
    """
    zone: Optional[ScwZone]
